define([
// dojo
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/aspect",
    "dojo/dom-class",
    "dojo/dom-geometry",
    "dojo/on",
    "dojo/when",

// epi
    "epi/shell/command/_WidgetCommandBinderMixin",

// epi-cms
    "./_ProjectView",
    "./viewmodels/ProjectComponentViewModel",

// template
    "dojo/text!./templates/ProjectComponent.html",

// Resources
    "epi/i18n!epi/cms/nls/episerver.cms.components.project",

// Widgets in template
    "dijit/DropDownMenu",
    "dijit/form/Button",
    "dijit/form/DropDownButton",
    "dijit/Toolbar",
    "./ProjectItemList",
    "./ProjectSelector",
    "./PublishMenu"
],

function (
// dojo
    declare,
    lang,
    aspect,
    domClass,
    domGeometry,
    on,
    when,

// epi
    _WidgetCommandBinderMixin,

// epi-cms
    _ProjectView,
    ProjectComponentViewModel,

// template
    template,

// Resources
    res
) {
    function show(list) {
        list.forEach(function (node) { domClass.remove(node, "dijitHidden"); });
    }

    function hide(list) {
        list.forEach(function (node) {  domClass.add(node, "dijitHidden"); });
    }

    var bindingMap = lang.delegate(_ProjectView.prototype.modelBindingMap, { placeHolder : ["placeHolder"] });

    return declare([_ProjectView, _WidgetCommandBinderMixin], {
        // summary:
        //
        // tags:
        //      internal

        "class": "epi-project-component",

        modelBindingMap: {
            "projectItemQuery": ["projectItemQuery"],
            "projectItemSortOrder": ["projectItemSortOrder"],
            "selectedProject": ["selectedProject"],
            "created": ["created"],
            "createdBy": ["createdBy"],
            "createProjectInfo": ["createProjectInfo"],
            "dndEnabled": ["dndEnabled"],
            "placeholderState": ["placeholderState"],
            "notificationMessage": ["notificationMessage"],
            "projectStatus": ["projectStatus"],
            "contentLanguage": ["contentLanguage"]
        },

        res: res,

        templateString: template,

        layout: function () {
            this.inherited(arguments);

            this.itemList.resize(this._contentBox, this._contentBox);
        },

        _createViewModel: function() {
            // summary:
            //    Setting up the view model
            // tags:
            //    private

            return new ProjectComponentViewModel();
        },

        _projectSelectorSortChanged: function(sortOrder) {
            this.model.set("projectSortOrder", sortOrder);
        },

        _projectSelectorChange: function (project) {
            this.model.set("selectedProject", project);
        },

        _newProjectButtonClickHandler: function () {
            this.model.namedCommands.addProject.execute();
        },

        _setCreateProjectInfoAttr: { node: "createProjectInfoNode", type: "innerText" },

        _setSelectedProjectAttr: function (value) {
            this.projectSelector.set("value", value);
        },

        _setPlaceholderStateAttr: function (value) {
            // summary:
            //    Sets the placeholderState
            // tags:
            //    private

            if (this.get("_placeholderState") === value) { return; }

            this._set("_placeholderState", value);

            switch (value) {

                case "noProjects":
                    show([this.placeholderNode]);
                    hide([this.projectSelector.domNode, this.publishContainer.domNode]);

                    break;

                case "noSelectedProject":
                    show([this.projectSelector.domNode, this.placeholderNode]);
                    hide([this.publishContainer.domNode]);

                    break;

                default:
                    show([this.projectSelector.domNode, this.publishContainer.domNode]);
                    hide([this.placeholderNode]);

            }
        }
    });
});
