define([
// dojo
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/aspect",
    "dojo/dom-geometry",
    "dojo/dom-class",
    "dojo/keys",
    "dojo/on",
    "dojo/when",

// dijit
    "dijit/_WidgetsInTemplateMixin",
    "dijit/layout/_LayoutWidget",
    "dijit/focus",
    "dijit/_CssStateMixin",
    "dijit/Tooltip",

// dgrid
    "dgrid/Keyboard",
    "dgrid/OnDemandList",
    "dgrid/Selection",
    "dgrid/extensions/DijitRegistry",

// epi
    "epi/shell/dnd/Target",
    "epi/shell/dgrid/Formatter",
    "epi/shell/widget/_FocusableMixin",
    "epi/shell/widget/_SectionedTemplatedMixin",
    "epi/shell/widget/ContextMenu",
    "epi-cms/extension/events",
    "epi-cms/dgrid/listItemFormatters",
    "epi-cms/dgrid/WithContextMenu",

// Resources
    "epi/i18n!epi/cms/nls/episerver.cms.components.project",

// template
    "dojo/text!./templates/ProjectItemList.html"

],
function (
// dojo
    declare,
    lang,
    aspect,
    domGeometry,
    domClass,
    keys,
    on,
    when,

// dijit
    _WidgetsInTemplateMixin,
    _LayoutWidget,
    focusUtil,
    _CssStateMixin,
    Tooltip,

// dgrid
    Keyboard,
    OnDemandList,
    Selection,
    DijitRegistry,

// epi
    Target,
    Formatter,
    _FocusableMixin,
    _SectionedTemplatedMixin,
    ContextMenu,
    events,
    listItemFormatters,
    WithContextMenu,

// Resources
    res,

// template
    template
) {

    return declare([_LayoutWidget, _SectionedTemplatedMixin, _WidgetsInTemplateMixin, _CssStateMixin, _FocusableMixin], {
        // summary:
        //
        // tags:
        //      internal

        contentLanguage: null,

        commandProvider: null,

        dndEnabled: false,

        sections: ["toolbarSection", "listContainer"],

        templateString: template,

        // totalItems: [public] int
        //      Indicates total items in the list
        totalItems: 0,

        // listItemType: [public] String
        //      What type of formatting to use from listItemFormatters
        listItemType: "card",

        res: res,

        _listClass: declare([OnDemandList, Formatter, Selection, Keyboard, WithContextMenu, DijitRegistry]),

        _list: null,

        // _narrowClass: String
        //      The width threshold in pixels
        _narrowThreshold: 570,

        // _narrowClass: String
        //      Class to apply to the widget domNode when it is too narrow.
        _narrowClass: "epi-card-list--narrow",

        buildRendering: function () {
            this.inherited(arguments);
            this._setupList();
        },

        focus: function() {
            // summary:
            //      Set focus to the project item list.
            // tags:
            //      public
            if (this._list) {
                this._list.focus();
            }
        },

        layout: function () {

            var toolbarHeight, size;

            if (!this._contentBox) {
                return;
            }

            toolbarHeight = domGeometry.getMarginBox(this.toolbar.domNode).h + domGeometry.getMarginBox(this.notificationBarNode).h;
            size = lang.mixin({}, this._contentBox, { h: this._contentBox.h - toolbarHeight });

            domGeometry.setMarginBox(this.listContainer, size);
            this._list.resize();

            domClass.toggle(this._list.domNode, this._narrowClass, size.w < this._narrowThreshold);
        },

        clearSelection: function() {
            // summary:
            //      Clears the selection in the list.
            // tags:
            //      public
            this._list.clearSelection();
            this.emit("selection-changed", { items: [] });
        },

        refresh: function () {
            // summary:
            //      Refreshes the itemlist
            // tags:
            //      public

            this._list.refresh({ keepScrollPosition: true });
        },

        setSelection: function(projectItems) {
            // summary:
            //      Sets the selection in the list to the given project items.
            // tags:
            //      public
            this._list.clearSelection();
            projectItems.forEach(function(item) { this._list.select(item); }, this);
            this.emit("selection-changed", { items: projectItems });
        },

        updateQuery: function (store, query) {

            var message = !query && this.res.noquery ? this.res.noquery : this.res.nodata;

            this._list.set("noDataMessage", message);
            this._list.set("store", store, query);
        },

        _setupList: function () {
            // summary:
            //    Setting up the list
            // tags:
            //    private

            var list, target;

            listItemFormatters.contentLanguage = this.contentLanguage;

            list = new this._listClass({
                formatters: listItemFormatters[this.listItemType],
                commandCategory: "itemContext",
                deselectOnRefresh: false,
                farOffRemoval: Infinity,
                noDataMessage: this.res.nodata
            }, this.itemListNode);

            list.contextMenu.addProvider(this.commandSource);

            this.own(
                this._list = list,
                list.on("dgrid-refresh-complete", lang.hitch(this, function (evt) {
                    when(evt.results.total, lang.hitch(this, function (total) {
                        this.set("totalItems", total);
                    }));
                })),
                list.on("dgrid-select, dgrid-deselect", lang.hitch(this, function(evt) {
                    var items = [];
                    Object.keys(list.selection).forEach(function(key) {
                        if (list.selection[key]) {
                            var item = list.row(key).data;
                            items.push(item);
                        }
                    });

                    this.emit("selection-changed", { items: items });
                })),
                list.on(".dgrid-row:dblclick", lang.hitch(this, function (evt) {
                    var item = this._list.row(evt).data;
                    this.emit("itemaction", { item: item });
                })),
                list.on(".dgrid-content:click", lang.hitch(this, function(e) {
                    if (e.target == list.contentNode) {
                        this.clearSelection();
                    }
                })),
                list.addKeyHandler(keys.ENTER, lang.hitch(this, function (evt) {
                    var item = this._list.row(evt).data;
                    this.emit("itemaction", { item: item });
                })),
                list.on(on.selector(".dgrid-row", events.keys.del), lang.hitch(this, function (evt) {
                    var item = this._list.row(evt).data;
                    this.emit("itemremove", { item: item });
                })),
                target = new Target(this.overlayNode, {
                    createItemOnDrop: false,
                    accept: ["episerver.core.icontentdata.reference"]
                }),
                aspect.after(target, "onDropData", lang.hitch(this, this._onDropData), true),
                // This class is used to enable showing the overlayNode of a list if list is empty (project gadget support)
                this.watch("totalItems", lang.hitch(this, function (name, oldValue, newValue) {
                    domClass.toggle(this.overlayNode, "epi-project-list-area__dnd-area--empty-list", newValue === 0);
                }))
            );
        },

        _onDropData: function (dndItems) {
            var items = dndItems.map(function (item) { return item.data; });
            this.emit("itemsdropped", { items: items });
        },

        _setDndEnabledAttr: function (value) {
            this._set("dndEnabled", value);
            domClass.toggle(this.overlayNode, "dijitHidden", !value);
        },

        _setNotificationMessageAttr: function (value) {
            this.notificationTextNode.textContent = value;
            domClass.toggle(this.notificationBarNode, "dijitHidden", !value);
            this.layout();
        },

        _setSortOrderAttr: function (value) {
            //Set the sort if it has changed
            if(this._list.get("sort") !== value) {
                this._list.set("sort", value);
            }
        },

        _setContentLanguageAttr: function (value) {
            this._set("contentLanguage", value);
            listItemFormatters.contentLanguage = value;
        }
    });
});
