define([
	"dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/topic",
    "dojo/when",
    "epi/dependency",

	 "./_ProjectViewModel"
], function (
//dojo
	declare,
	lang,
	topic,
    when,

//epi
    dependency,
    _ProjectViewModel
) {
	return declare([_ProjectViewModel], {
        // summary:
        //
        // tags:
        //      internal

        // contextHistory: [readonly] epi-cms/BackContextHistory
        //      The context history stack
        contextHistory: null,

        // isProjectOverviewActive: [public] Boolean
        //      Indicates if the overview is open
        isProjectOverviewActive: false,

        // _sortOrderProfileKey: [protected] String
        //      The profile key used to store the sort order
        _sortOrderProfileKey: "epi.project-overview-sort-order",

		postscript: function () {
			this.inherited(arguments);

            this.contextHistory = this.contextHistory || dependency.resolve("epi.cms.BackContextHistory");
			this.own(this.projectService.on("currentProjectChanged", lang.hitch(this, this.projectOverviewChanged)));
		},

        contextChanged: function (/*Object*/context, /*Object*/callerData) {
            // summary:
            //      When context changes
            // tags:
            //      protected 
            this.inherited(arguments);

            if(context && context.type === "epi.cms.project") {
                var projectId = parseInt(context.id, 10);
                when(this.projectStore.refresh(projectId), lang.hitch(this,function(currentProject) {
                    this.set("selectedProject", currentProject);
                    this.emit("clear-selection");

                    //Workaround: when we refresh overview and selectedProject does an early exit and we want the list to refresh anyway.
                    this.emit("refresh");
                }));
            } 
        },

        projectOverviewChanged: function (project) {
            // summary:
            //      When the project overview changes
            // project: Object
            //      Project      
            // tags:
            //      public   
                 
            var active = this.get("isProjectOverviewActive"),
                newProjectUri;

            // id is empty when selecting None.
            if(active) {
                if (project && project.id) {
                    newProjectUri = "epi.cms.project:///" + project.id;
                    topic.publish("/epi/shell/context/request", { uri: newProjectUri }, { sender: this });
                } else {
                    this.requestPreviousContext();
                }
            }
        },

        requestPreviousContext: function () {
            // summary:
            //      Requests the previous context and publish context change
            // tags:
            //      public  
            var context = this.contextHistory && this.contextHistory.getPreviousContent();
            if (context && context.uri) {
                topic.publish("/epi/shell/context/request", context, { sender: this });
            } 
        },

        _createProjectItemCommands: function () {

            var commands = this.inherited(arguments);

            commands.sortProjectItems.set({
                category: "projectButton",
                iconClass: "epi-iconSort"
            });

            return commands;
        }

	});

});