﻿define([
    "dojo/_base/declare",
    "dojo/_base/kernel",
    "dojo/_base/lang",
    "dojo/aspect",
    "dojo/topic",
    "dojo/when",
    "dojo/dom-class",

// dijit
    "dijit/DropDownMenu",
    "dijit/form/DropDownButton",
    "dijit/focus",

// epi
    "epi/dependency",
    "epi-cms/widget/ViewLanguageSelector",

// resources
    "epi/i18n!epi/cms/nls/episerver.cms.widget.viewlanguagebutton"
], function (
    declare,
    kernel,
    lang,
    aspect,
    topic,
    when,
    domClass,

// dijit
    DropDownMenu,
    DropDownButton,
    focusUtil,

// epi
    dependency,
    ViewLanguageSelector,

// resources
    resources
) {
    kernel.deprecated("epi-cms/widget/ViewLanguageButton should no longer be used",
        "use the epi-cms/contentediting/command/LanguageSelection command instead", "9.0");

    return declare([DropDownButton], {
        // tags:
        //      internal

        // _contextMenu: [private] dijit.DropDownMenu
        //    The context menu for the button.
        _contextMenu: null,

        // baseClass: [public] String
        //    CSS base classes for the button.
        baseClass: "epi-mediumButton epi-chromelessButton",

        // iconClass: [public] String
        //    CSS class that adds the button icon.
        iconClass: "",

        // contextMenuProperties [public] Object
        //    Context menu properties.
        contextMenuProperties: {},

        // _viewLanguageSelector: [private] Object
        //    The preview language selector widget.
        _viewLanguageSelector: null,

        // _viewSettingsManager: [private] Object
        //    The view manager
        _viewSettingsManager: null,

        postMixInProperties: function () {
            // summary:
            //    Mixin the widget properties.
            //
            // tags:
            //    protected

            this.inherited(arguments);

            lang.mixin(this.contextMenuProperties, {
                "class": "epi-menu--inverted"
            });

            this.profile = this.profile || dependency.resolve("epi.shell.Profile");
        },

        postCreate: function () {
            // summary:
            //    Raised after the widget is created. It will create
            //    the widget's context menu and add its children.
            //
            // tags:
            //    protected

            this.inherited(arguments);

            //Get the content language
            when(this.profile.getContentLanguage(), lang.hitch(this, function(language) {

                this._viewSettingsManager = dependency.resolve("epi.viewsettingsmanager");
                var defaultItem = {
                    languageId: language
                },
                checkedItem = this._viewSettingsManager.getSettingProperty("viewlanguage") || defaultItem.languageId;

                if (!this.dropDown) {

                    this.dropDown = DropDownMenu(this.contextMenuProperties);

                    this.own(this.dropDown);

                    this._viewLanguageSelector = new ViewLanguageSelector({
                        defaultItem: defaultItem,
                        checkedItem: checkedItem
                    });
                    this.own(aspect.after(this._viewLanguageSelector, "onItemSelected", lang.hitch(this, "_onViewLanguageSelected"), true));

                    this._addChildren(this._viewLanguageSelector);

                    when(this._viewLanguageSelector.fetchData(), lang.hitch(this, function(result) {
                        if (this._viewLanguageSelector.get("items").length === 0) {
                            domClass.add(this.domNode, "dijitHidden");
                            return;
                        }
                        this._changeView();
                    }));
                }

                this.own(aspect.after(this.dropDown, "onOpen", lang.hitch(this, function(e) {
                    focusUtil.focus(this._viewLanguageSelector.menuNode.domNode);
                }), true));

                this.own(topic.subscribe("/epi/cms/contentdata/updated", lang.hitch(this, function() {
                    //when the language settings dialog is closed, get the current site
                    this._languageSettingsChanged();
                })));
            }));
        },

        _languageSettingsChanged: function () {
            // summary:
            //      Update available languages for button
            // tags:
            //      private

            when(this._viewLanguageSelector.fetchData(), lang.hitch(this, function (result) {
                var selectedItem = this._viewLanguageSelector.get("currentSelectedItem");
                this._changeView(selectedItem || this._viewLanguageSelector.defaultItem);
            }));
        },

        _updateButton: function (/*Object*/languageItem) {
            // summary:
            //      Update button display text.
            // tags:
            //      private

            var selectedItem = languageItem || this._viewLanguageSelector.get("currentSelectedItem") || this._viewLanguageSelector.defaultItem;
            var tooltiptext = lang.replace(resources.tooltip, { viewlanguage: selectedItem ? selectedItem.name : null });
            var labeltext = selectedItem && selectedItem.urlSegment ? selectedItem.urlSegment.toUpperCase() : null;
            var oldLabelText = this.get("label");
            this.set("title", tooltiptext);
            this.set("label", labeltext);

            if (oldLabelText != labeltext) {
                // Resize view settings menu's expando button
                this.onResize();
            }
        },

        onResize: function () {
            // summary:
            //    Raised when a item change.
            //
            // tags:
            //    protected event
        },

        _isDefaultSelection: function () {
            // summary:
            //    Checks if the selected items on both widgets are the default items.
            //
            // tags:
            //    private

            if (this._viewLanguageSelector.get("currentSelectedItem")) {
                return this._viewLanguageSelector.isDefaultItemSelected();
            }

            return true;
        },

        _onViewLanguageSelected: function (/*Object*/language) {
            // summary:
            //    Raised when a preview language is selected.
            //
            // language:
            //   The selected preview language
            //
            // tags:
            //    private event

            this._changeView(language);

            // Close alway view language menu when item has selected
            this.closeDropDown();
        },

        _changeView: function (/*Object*/language) {
            // summary:
            //    Change the view to the given resolution.
            //
            // language:
            //    preview language object
            //
            // tags:
            //    private

            if (language) {
                topic.publish("/epi/cms/action/viewsettingvaluechanged", "viewlanguage", language.languageId);
            }

            this._viewLanguageSelector.checkItem(language);
            this._activeButton(!this._isDefaultSelection());
            this._updateButton(language);
        },

        _addChildren: function (/*Object*/item) {
            // summary:
            //    Populate the context menu with the configured items.
            //
            // item:
            //    The item that will be added to the dropdown
            //
            // tags:
            //    private

            this.dropDown.addChild(item);
        },

        _activeButton: function (/*Boolean*/active) {
            // summary:
            //      Set active state for button widget
            //
            // active:
            //    States if the button will be checked or not.
            //
            // tags:
            //      private
            if (!this.dropDown) {
                return;
            }

            domClass.toggle(this.domNode, "dijitChecked", active);
            domClass.toggle(this.domNode, "epi-expandoButtonLangSelected", active);
        }
    });
});