﻿define([
    "dojo/_base/declare",
    "dojo/_base/lang",

    "dijit/Menu",
    "dijit/form/Button",
    "dijit/form/DropDownButton",

    "../_CommandModelBindingMixin",
    "./_Builder",
    "./OptionsBuilderMixin"

], function (
    declare,
    lang,

    Menu,
    Button,
    DropDownButton,

    _CommandModelBindingMixin,
    _Builder,
    OptionsBuilderMixin
) {

    var dropDownClass = declare([DropDownButton, _CommandModelBindingMixin], {
        onClick: function () {}
    });

    return declare([_Builder], {
        // summary:
        //      Builds a button widget and connects events based on a command object.
        // 
        // tags:
        //      internal xproduct

        // optionClass: string
        //      default css class for the dropdown created for the options
        // tags:
        //      internal
        optionClass: "",

        // optionItemClass: string
        //      default css class for the dropdown created for the options
        // tags:
        //      internal
        optionItemClass: "",

        // _buttonClass: [protected] class
        //      The class used for representing a button bound to a command
        _buttonClass: declare([Button, _CommandModelBindingMixin]),

        _dropDownClass: dropDownClass,

        _optionsMenuClass: declare([Menu, OptionsBuilderMixin]),

        _create: function (/*epi/shell/command/_Command*/command) {
            // tags:
            //		protected

            var hasOptions = command.get("options") instanceof Array,
                options = this._createOptions(command);

            if (hasOptions) {

                return new this._dropDownClass(lang.mixin({
                    "class": "epi-chromeless epi-mediumButton",
                    dropDown: new this._optionsMenuClass({
                        "class": this.optionClass,
                        model: command,
                        optionItemClass: this.optionItemClass
                    })
                }, options));
            }

            return new this._buttonClass(options);
        },

        _createOptions : function (/*epi/shell/command/_Command*/command) {
            // tags:
            //		protected

            return lang.mixin({
                model: command
            }, this.settings);
        }
    });
});